# This file is encoded in ShiftJIS.
die "This file is not encoded in ShiftJIS.\n" if q{} ne "\x82\xa0";

use Sjis;

local $^W = 1;

if (@ARGV < 2) {
    die <<'END';
s@:

perl grepdir.pl aaa dt
perl grepdir.pl \  dt
END
}

&grepdir(@ARGV);

exit 0;

sub grepdir ($$) {
    my($pat,$dir) = @_;
    my($node);

# Cӏ1
#   opendir(D,$dir);
#   my @nodes = grep (!/^\./, readdir(D));
#   closedir(D);
    my @nodes = split(/\n/,`dir /b $dir 2>NUL`);

    # --------------------------------------------------
    foreach $node (@nodes) {

# Cӏ2
#       my $path="$dir/$node";
        my $path="$dir\\$node";

        print "!!$path!!\n";

        if ( -f $path ) {
            grepfile($pat,$path);
        }
        elsif( -d $path) {
            &grepdir($pat,$path);
        }
        else {
            print STDERR "skip:$path\n";
        }
    }
}

sub grepfile ($$) {
    my($pat,$file) = @_;
    open(IN,$file) or die "Error:open($file):$!\n";
    while (<IN>) {
        chomp;

# Cӏ3
#       print "$file:$_\n" if (/$pat/);
        print "$file:$_\n" if (/\Q$pat\E/);
    }
}

__END__

WindowsPerl 5.8/5.10gȂ
http://www.aritia.org/hizumi/perl/perlwin.html

́uŏЉXNvg̃Tvv grepdir.pl 𗘗pĂ܂B

: K\w肵āCw肵fBNgz̃t@CoR[hĂB

R}h`: perl grepdir.pl {p^[} {fBNg}

̂悤ȃeXgpӂB

C:\TEMP\TP> tree /F dt
tH_ pẌꗗ: {[ vvvvv_vvvvvvvvv
{[ VAԍ vvvv-ssss ł
C:\TEMP\TP\DT
alphabet
      alpha.txt
      sjis.txt

{
       alpha.txt
       sjis.txt

 perl ɂĎsƁĈ悤ɂȂB

C:\TEMP\TP\DT>perl grepdir.pl aaa dt
!!dt/alphabet!!
!!dt/alphabet/alpha.txt!!
dt/alphabet/alpha.txt:aaa
!!dt/alphabet/sjis.txt!!
dt/alphabet/sjis.txt:aaa
!!dt/{!!
!!dt/{/alpha.txt!!
dt/{/alpha.txt:aaa
!!dt/{/sjis.txt!!
dt/{/sjis.txt:aaa

C:\TEMP\TP\DT>perl grepdir.pl \ dt
!!dt\alphabet!!
!!dt\alphabet\alpha.txt!!
!!dt\alphabet\sjis.txt!!
dt\alphabet\sjis.txt:\
!!dt\{!!
!!dt\{\alpha.txt!!
!!dt\{\sjis.txt!!
dt\{\sjis.txt:\

Ȃ΂ȂȂƂ́Cȉ̂悤ȂƂɂȂB

Cӏ1
  MS-DOS, Microsoft Windows ̓{ŏ opendir, readdir, closedir 
  gp̂͂ƂƖ肪̂ňȉ̂悤 dir R}hgƂ悢B

    ----------------------------------------------
    opendir(D,$dir);
    my @nodes = grep (!/^\./, readdir(D));
    closedir(D);
    ----------------------------------------------
                 
    ----------------------------------------------
    my @nodes = split(/\n/,`dir /b $dir 2>NUL`);
    ----------------------------------------------

Cӏ2
  Windows ł̃fBNg̋؂ \ Ȃ̂ \ gB

    ----------------------------------------------
    my $path="$dir/$node";
    ----------------------------------------------
                 
    ----------------------------------------------
    my $path="$dir\\$node";
    ----------------------------------------------

Cӏ3
  K\ɕϐLqAϐɊi[Ăê̂Ƀ}b`
  ̂ł \Q ... \E ň͂ޕKvB
  (ƂCӏ2 /  \ ɕύXĂ܂߂B)

    ----------------------------------------------
    print "$file:$_\n" if (/$pat/);
    ----------------------------------------------
                 
    ----------------------------------------------
    print "$file:$_\n" if (/\Q$pat\E/);
    ----------------------------------------------

ȏ
